# {{ project_identifier }}

## Setup

### Install requirements

```bash
pip install -r requirements.txt
```

### Getting started guide

```bash
python main.py
```

### Running tests

```bash
piccolo tester run
```
